/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: layfrm.hxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: rt $ $Date: 2006/10/27 22:46:34 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/
#ifndef _LAYFRM_HXX
#define _LAYFRM_HXX

#include "frame.hxx"
namespace binfilter {

class SwCntntFrm;
class SwFlowFrm;
class SwFmtCol;
struct SwCrsrMoveState;
class SwFrmFmt;
class SwBorderAttrs;
class SwFmtFrmSize;
class SwCellFrm;

class SwLayoutFrm: public SwFrm
{
	//Der verkappte SwFrm
	friend class SwFlowFrm;
	friend class SwFrm;

		//Hebt die Lower waehrend eines Spaltenumbaus auf.
	friend SwFrm* SaveCntnt( SwLayoutFrm *, SwFrm * );
	friend void   RestoreCntnt( SwFrm *, SwLayoutFrm *, SwFrm *pSibling );

	//entfernt leere SwSectionFrms aus einer Kette
	friend SwFrm* SwClearDummies( SwFrm* pFrm );

	void CopySubtree( const SwLayoutFrm *pDest );
protected:
	virtual void Format( const SwBorderAttrs *pAttrs = 0 );
	virtual void MakeAll();

	SwFrm			*pLower;

    virtual SwTwips ShrinkFrm( SwTwips, SZPTR
							   BOOL bTst = FALSE, BOOL bInfo = FALSE );
    virtual SwTwips GrowFrm  ( SwTwips, SZPTR
							   BOOL bTst = FALSE, BOOL bInfo = FALSE );

	long CalcRel( const SwFmtFrmSize &rSz, BOOL bWidth ) const;

public:
	void PaintSubsidiaryLines( const SwPageFrm*, const SwRect& ) const;

		//Proportionale Groessenanpassung der untergeordneten.
	void ChgLowersProp( const Size& rOldSize );

    void AdjustColumns( const SwFmtCol *pCol, BOOL bAdjustAttributes,
                        BOOL bAutoWidth = FALSE);

	void ChgColumns( const SwFmtCol &rOld, const SwFmtCol &rNew,
		const BOOL bChgFtn = FALSE );


		//Painted die Column-Trennlinien fuer die innenliegenden Columns.


	virtual void Cut();
	virtual void Paste( SwFrm* pParent, SwFrm* pSibling = 0 );

		//sucht den dichtesten Cntnt zum SPoint, wird bei Seiten, Flys und Cells
		//benutzt wenn GetCrsrOfst versagt hat.
	const SwCntntFrm* GetCntntPos( Point &rPoint, const BOOL bDontLeave,
								   const BOOL bBodyOnly = FALSE,
								   const BOOL bCalc = FALSE,
								   const SwCrsrMoveState *pCMS = 0,
								   const BOOL bDefaultExpand = TRUE ) const;

	SwLayoutFrm( SwFrmFmt* );
	~SwLayoutFrm();

	const SwFrm *Lower() const { return pLower; }
		  SwFrm *Lower()	   { return pLower; }
	const SwCntntFrm *ContainsCntnt() const;
	inline SwCntntFrm *ContainsCntnt();
	const SwCellFrm *FirstCell() const;
	inline SwCellFrm *FirstCell();
	const SwFrm *ContainsAny() const;
	inline SwFrm *ContainsAny();
	BOOL IsAnLower( const SwFrm * ) const;

	const SwFrmFmt *GetFmt() const { return (const SwFrmFmt*)GetDep(); }
		  SwFrmFmt *GetFmt()	   { return (SwFrmFmt*)GetDep(); }
	void 			SetFrmFmt( SwFrmFmt* );

	//Verschieben der Ftns aller Lower - ab dem StartCntnt.
	//TRUE wenn mindestens eine Ftn verschoben wurde.
	//Ruft das Update der Seitennummer wenn bFtnNums gesetzt ist.
	BOOL MoveLowerFtns( SwCntntFrm *pStart, SwFtnBossFrm *pOldBoss,
						SwFtnBossFrm *pNewBoss, const BOOL bFtnNums );

	//Sorgt dafuer, dass innenliegende Flys noetigenfalls zum clippen bzw.
	//reformatieren invalidiert werden.
	void NotifyFlys();

	//Invalidiert diejenigen innenliegenden Frames, deren Breite und/oder
	//Hoehe Prozentual berechnet werden. Auch Rahmen, die an this oder an
	//innenliegenden verankert sind werden ggf. invalidiert.
    void InvaPercentLowers( SwTwips nDiff = 0 );

	//Gerufen von Format fuer Rahmen und Bereichen mit Spalten.
	void FormatWidthCols( const SwBorderAttrs &, const SwTwips nBorder,
						  const SwTwips nMinHeight );

	// InnerHeight returns the height of the content and may be bigger or
	// less than the PrtArea-Height of the layoutframe himself
	SwTwips InnerHeight() const;

    /** method to check relative position of layout frame to
        a given layout frame.

        OD 08.11.2002 - refactoring of pseudo-local method <lcl_Apres(..)> in
        <txtftn.cxx> for #104840#.

        @param _aCheckRefLayFrm
        constant reference of an instance of class <SwLayoutFrm> which
        is used as the reference for the relative position check.

        @author OD

        @return true, if <this> is positioned before the layout frame <p>
    */
    bool IsBefore( const SwLayoutFrm* _pCheckRefLayFrm ) const;
};

//Um doppelte Implementierung zu sparen wird hier ein bischen gecasted
inline SwCntntFrm* SwLayoutFrm::ContainsCntnt()
{
	return (SwCntntFrm*)(((const SwLayoutFrm*)this)->ContainsCntnt());
}

inline SwCellFrm* SwLayoutFrm::FirstCell()
{
	return (SwCellFrm*)(((const SwLayoutFrm*)this)->FirstCell());
}

inline SwFrm* SwLayoutFrm::ContainsAny()
{
	return (SwFrm*)(((const SwLayoutFrm*)this)->ContainsAny());
}

// Diese SwFrm-inlines sind hier, damit frame.hxx nicht layfrm.hxx includen muss
inline BOOL SwFrm::IsColBodyFrm() const
{
    return nType == FRMC_BODY && GetUpper()->IsColumnFrm();
}

inline BOOL SwFrm::IsPageBodyFrm() const
{
    return nType == FRMC_BODY && GetUpper()->IsPageFrm();
}

} //namespace binfilter
#endif	//_LAYFRM_HXX
